//
//  MPScheme.h
//  mpos.core
//
//  Created by Leonid Popescu on 7/28/14.
//  Copyright (c) 2021 CyberSource Corporation. All Rights Reserved.
//

#import "MPPaymentDetails.h"
@import Foundation;

NS_ASSUME_NONNULL_BEGIN

@interface MPScheme : NSObject

@property (strong, readonly, nonatomic) NSArray<NSString *> *aids;
@property (strong, readonly, nonatomic) NSString *name;
@property (assign, readonly, nonatomic) BOOL magstripeSupported;

@property (assign, readonly, nonatomic) BOOL magstripeForcePin;
@property (assign, readonly, nonatomic) BOOL magstripeAllowPinBypass;
@property (assign, readonly, nonatomic) BOOL magstripeLimitServiceCodeChecks;

@property (strong, readonly, nonatomic) NSArray<NSNumber *> *magstripeCoBrandedBins;

@property (assign, readonly, nonatomic) MPPaymentDetailsScheme scheme;
@property (strong, readonly, nonatomic) NSString *string;


- (BOOL)isBinNumberInCoBrandedRange:(NSUInteger)coBrandedBinNumber;
- (BOOL)isBinNumberInRange:(NSUInteger)binNumber;
- (BOOL)isCompatibleWithAID:(NSData *)aid;

- (BOOL)isCompatibleWithTrack2:(NSString *)track2;
- (BOOL)isCoBrandCompatibleWithTrack2:(NSString *)track2;
- (BOOL)isCompatibleWithAccountNumber:(NSString *)accountNumber;
- (BOOL)isCoBrandCompatibleWithAccountNumber:(NSString *)accountNumber;

- (BOOL)isVisaFamily;
- (BOOL)isMastercardFamily;
- (BOOL)isDebit;
- (BOOL)isGiftCard;
- (BOOL)isWalletScheme;

- (NSString *)genericAIDFromAID:(NSString *)aid;

+ (NSDictionary<NSString *, MPScheme *> *)schemes;
+ (MPPaymentDetailsScheme)schemeEnumFromString:(NSString *)scheme;
+ (MPScheme *)schemeForSchemeEnum:(MPPaymentDetailsScheme)schemeEnum;
+ (MPScheme *)schemeFromString:(NSString *)scheme;
+ (NSArray<MPScheme *> *)schemesForSchemeEnums:(NSArray<NSNumber *> *)schemeEnums;

+ (MPScheme *)schemeForAccountNumber:(nullable NSString *)number;
+ (MPScheme *)schemeForAccountNumber:(nullable NSString *)number includeCoBranded:(BOOL)includeCoBranded;

+ (nullable MPScheme *)schemeForAID:(nullable NSData *)aid accountNumber:(nullable NSString *)accountNumber track2:(nullable NSString *)track2;

+ (MPScheme *)unknownScheme;

+ (nullable NSString *)displayNameForAID:(nullable NSData *)aid accountNumber:(nullable NSString *)accountNumber track2:(nullable NSString *)track2;


@end

NS_ASSUME_NONNULL_END
